package abchr;

import abchr.audio.Sample;

public abstract class AbstractABXTestFormatter implements ABXTestFormatter {
	// test formatting strings
	public static final String A_SAMPLE="$A_SAMPLE$";
	public static final String B_SAMPLE="$B_SAMPLE$";
	public static final String RESULT_LIST="$RESULT_LIST$";
	public static final String FINAL_RESULT="$FINAL_RESULT$";
	// trial formatting strings
	public static final String PASSFAIL_S="$PASSFAIL_S$";
	public static final String PASSFAIL_M="$PASSFAIL_M$";
	public static final String PASSFAIL_L="$PASSFAIL_L$";
	public static final String PVAL="$PVAL$";
	public static final String PVAL_EXT="$PVAL_EXT$";
	public static final String CORRECT="$CORRECT$";
	public static final String TOTAL="$TOTAL$";

	protected ABXTest test;

	protected String formattingString;
	protected String trialFormattingString;

	protected SampleFormatter sampleFormatter=new SampleFormatter() {
		public String format(Sample sample) {
			Object name=sample.getAttribute("name");
			if(name==null || !(name instanceof String)) {
				return sample.toString();
			} else {
				return (String)name;
			}
		}
	};

	protected AbstractABXTestFormatter(ABXTest test) {
		this.test=test;
	}

	public ABXTest getTest(){return test;}
	public void setTest(ABXTest test){this.test=test;}

	public String format() {
		String result=formattingString;
		result=simpleReplace(result,A_SAMPLE,sampleFormatter.format(test.getASample()));
		result=simpleReplace(result,B_SAMPLE,sampleFormatter.format(test.getBSample()));
		if(result.indexOf(RESULT_LIST)!=-1) {
			result=simpleReplace(result,RESULT_LIST,makeResultList());
		}
		if(result.indexOf(FINAL_RESULT)!=-1) {
			result=simpleReplace(result,FINAL_RESULT,makeFinalResult());
		}
		return result;
	}

	private String makeResultList() {
		String nl=System.getProperty("line.separator");
		StringBuffer buffer=new StringBuffer();
		int n=0,correct=0;
		boolean pass;
		float pval;
		for(int i=0;i<test.getNumTrials();i++) {
			n++;
			pass=test.getResult(i);
			if(pass){correct++;}
			pval=ABXTestImpl.pVal(n,correct);
			String trialString=formatTrial(n,correct,pval,pass);
			buffer.append(trialString);
			buffer.append(nl);
		}
		return buffer.toString();
	}

	protected String formatTrial(int n,int correct,float pval,boolean pass) {
		String trialString=trialFormattingString;
		trialString=simpleReplace(trialString,TOTAL,Integer.toString(n));
		trialString=simpleReplace(trialString,CORRECT,Integer.toString(correct));
		trialString=simpleReplace(trialString,PVAL,Float.toString(pval));
		trialString=simpleReplace(trialString,PVAL_EXT,formatPVal(pval));
		trialString=simpleReplace(trialString,PASSFAIL_S,pass?"p":"f");
		trialString=simpleReplace(trialString,PASSFAIL_M,pass?"pass":"fail");
		trialString=simpleReplace(trialString,PASSFAIL_L,pass?"passed":"failed");
		return trialString;
	}

	protected static String formatPVal(float pval) {
		return pval<0.001?"< 0.001":("= "+Float.toString(((int)(pval*1000))/1000.0f));
	}

	private String makeFinalResult() {
		return formatTrial(test.getNumTrials(),test.getCorrect(),test.pVal(),test.pVal()<0.05);
	}

	public void setFormattingString(String format){formattingString=format;}
	public void setTrialFormattingString(String format){trialFormattingString=format;}

	public void setSampleFormatter(SampleFormatter formatter) {
		this.sampleFormatter=formatter;
	}

	protected static String simpleReplace(String s,String t,String replacement) {
		int i;
		while((i=s.indexOf(t))!=-1) {
			s=s.substring(0,i)+replacement+s.substring(i+t.length());
		}
		return s;
	}
}