package abchr;

import abchr.audio.Sample;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.*;
import java.io.Serializable;
import java.io.IOException;
import java.util.Random;
import java.util.List;
import java.util.ArrayList;

public class ComparisonGroup implements Serializable {
	private static final Random random=new MersenneTwister();

	private final Sample[] samples=new Sample[2];
	private final int originalIndex;
	private final BoundedRangeModel[] ratings=new BoundedRangeModel[2];
	private final String[] comments=new String[2];

	private boolean blind=true;

	private transient List<ChangeListener> listeners=new ArrayList<ChangeListener>(2);

	public ComparisonGroup(Sample original,Sample encoded,boolean randomize) {
		originalIndex=randomize?(random.nextInt()>0?0:1):0;
		this.samples[originalIndex]=original;
		this.samples[(originalIndex+1)%2]=encoded;
		this.comments[0]="";
		this.comments[1]="";
		this.ratings[0]=new DefaultBoundedRangeModel(50,0,10,50);
		this.ratings[1]=new DefaultBoundedRangeModel(50,0,10,50);
		ratings[0].addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if(ratings[0].getValue()!=ratings[0].getMaximum()) {
					ratings[1].setValue(ratings[1].getMaximum());
				}
			}
		});
		ratings[1].addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if(ratings[1].getValue()!=ratings[1].getMaximum()) {
					ratings[0].setValue(ratings[0].getMaximum());
				}
			}
		});
	}

	public Sample getSample(int index){return samples[index];}

	public String getComment(int index){return comments[index];}
	public void setComment(int index,String value){comments[index]=value;notifyListeners();}

	public int getOriginalIndex(){return originalIndex;}
	public int getRatedBadIndex() {
		if(ratings[0].getValue()!=ratings[0].getMaximum()) {
			return 0;
		} else if(ratings[1].getValue()!=ratings[1].getMaximum()) {
			return 1;
		} else {
			return -1;
		}
	}

	public BoundedRangeModel getRating(int index){return ratings[index];}

	public boolean isBlind(){return blind;}
	public void setBlind(boolean blind){this.blind=blind;notifyListeners();}

	protected void notifyListeners() {
		ChangeEvent e=new ChangeEvent(this);
		for(ChangeListener l : listeners) {
			l.stateChanged(e);
		}
	}

	public void addListener(ChangeListener listener){listeners.add(listener);}
	public void removeListener(ChangeListener listener){listeners.remove(listener);}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		//System.out.println("Writing SampleConfigModule");
		ratings[0]=new DefaultBoundedRangeModel(ratings[0].getValue(),ratings[0].getExtent(),ratings[0].getMinimum(),ratings[0].getMaximum());
		ratings[1]=new DefaultBoundedRangeModel(ratings[1].getValue(),ratings[1].getExtent(),ratings[1].getMinimum(),ratings[1].getMaximum());
		out.defaultWriteObject();
		//System.out.println("Done writing SampleConfigModule");
	}

	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException {
		in.defaultReadObject();
		listeners=new ArrayList<ChangeListener>(2);
	}
}