package abchr;

import java.text.DateFormat;
import java.util.Locale;
import java.util.Date;

public class DetailedABXTestFormatter extends AbstractABXTestFormatter {
	private static final String TIME="$TIME$";
	private static final String RANGE_BEGIN="$RANGE_BEGIN$";
	private static final String RANGE_END="$RANGE_END$";
	private static final String RANGE="$RANGE$";
	private static final String RANGE_COND="$RANGE_COND$";

	public static interface RangeFormat {
		public String format(int rangeBegin,int rangeEnd);
	}

	public static class SimpleRangeFormatter implements RangeFormat {
		public String formatTime(int time) {
			int msecs=time%1000;
			int secs=(time/1000)%60;
			int mins=(time/60000);
			StringBuffer buffer=new StringBuffer();
			if(mins!=0) {
				if(mins<10){buffer.append('0');}
				buffer.append(mins);
				buffer.append(':');
			}
			if(secs<10){buffer.append('0');}
			buffer.append(secs);
			buffer.append('.');
			if(msecs<100){buffer.append('0');}
			if(msecs<10){buffer.append('0');}
			buffer.append(msecs);
			return buffer.toString();
		}

		public String format(int rangeBegin,int rangeEnd) {
			return formatTime(rangeBegin)+'-'+formatTime(rangeEnd);
		}
	}

	protected ABXTest test;

	protected DateFormat dateFormatter=DateFormat.getTimeInstance(DateFormat.MEDIUM,Locale.US);
	protected RangeFormat rangeFormatter=new SimpleRangeFormatter();

	public DetailedABXTestFormatter(ABXTest test) {
		super(test);
		this.test=test;
	}

	public void setTest(ABXTest test) {
		super.setTest(test);
		this.test=test;
	}

	public void setDateFormatter(DateFormat dateFormatter) {
		this.dateFormatter=dateFormatter;
	}

	public void setRangeFormatter(RangeFormat rangeFormatter) {
		this.rangeFormatter=rangeFormatter;
	}

	protected String formatTrial(int n,int correct,float pval,boolean pass) {
		String result=super.formatTrial(n,correct,pval,pass);
		ABXTrial trial=test.getTrial(n-1);
		result=simpleReplace(result,TIME,dateFormatter.format(new Date(trial.getTimeStamp())));
		int rangeBegin=trial.getRangeBegin();
		int rangeEnd=trial.getRangeEnd();
		result=simpleReplace(result,RANGE_BEGIN,Float.toString(rangeBegin/1000.0f));
		result=simpleReplace(result,RANGE_END,Float.toString(rangeEnd/1000.0f));
		if(result.indexOf(RANGE)!=-1) {
			result=simpleReplace(result,RANGE,rangeFormatter.format(rangeBegin,rangeEnd));
		}
		if(result.indexOf(RANGE_COND)!=-1) {
			if(n==1 || test.getTrial(n-2).getRangeBegin()!=rangeBegin || test.getTrial(n-2).getRangeEnd()!=rangeEnd) {
				result=simpleReplace(result,RANGE_COND,rangeFormatter.format(rangeBegin,rangeEnd));
			} else {
				result=simpleReplace(result,RANGE_COND,"");
			}
		}
		return result;
	}
}