package abchr;

import abchr.crypto.ElGamalPublicKey;
import org.jdom.Document;
import org.jdom.Element;

import java.math.BigInteger;
import java.io.IOException;

public class EncryptionKeyModule extends AbstractProjectModule implements ConfigModule {
	private ElGamalPublicKey encryptionKey;

	public ElGamalPublicKey getEncryptionKey(){return encryptionKey;}
	public void setEncryptionKey(ElGamalPublicKey encryptionKey){this.encryptionKey=encryptionKey;}

	public EncryptionKeyModule(ElGamalPublicKey encryptionKey) {
		this.encryptionKey=encryptionKey;
	}

	public Element toResultsXML() {
		return toXML();
	}

	public static ProjectModule createInstance(Document doc) {
		Element keyElement=doc.getRootElement().getChild("encryptionkey");
		if(keyElement==null){return new EncryptionKeyModule(null);}
		BigInteger modulus=new BigInteger(keyElement.getChild("modulus").getText(),32);
		BigInteger generator=new BigInteger(keyElement.getChild("generator").getText(),32);
		BigInteger publicKey=new BigInteger(keyElement.getChild("publickey").getText(),32);
		return new EncryptionKeyModule(new ElGamalPublicKey(modulus,generator,publicKey));
	}

	public org.jdom.Element toXML() {
		if(encryptionKey==null){return null;}
		org.jdom.Element root=new org.jdom.Element("encryptionkey");
		root.addContent(XMLUtils.createJDomTextElement("modulus",encryptionKey.getModulus().toString(32)));
		root.addContent(XMLUtils.createJDomTextElement("generator",encryptionKey.getGenerator().toString(32)));
		root.addContent(XMLUtils.createJDomTextElement("publickey",encryptionKey.getPublicKey().toString(32)));
		return root;
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		//System.out.println("Writing EncryptionKeyModule");
		out.defaultWriteObject();
		//System.out.println("Done writing EncryptionKeyModule");
	}
}
