package abchr;

import org.jdom.Document;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

public class ProjectLoader {
	private static Class[] classes;
	private static Method[] factoryMethods;
	private static int[][] parameterIndices;

	public static TestProject load(Document document) throws ParseException {
		ProjectModule[] modules=new ProjectModule[classes.length];
		Object[] parameters;
		Method factoryMethod;
		boolean done=false;
		while(!done) {
			done=true;
			outer: for(int i=0;i<factoryMethods.length;i++) {
				if(modules[i]!=null){continue;}
				factoryMethod=factoryMethods[i];
				parameters=new Object[parameterIndices[i].length];
				for(int j=0;j<parameters.length;j++) {
					if(parameterIndices[i][j]==-1) {
						parameters[j]=document;
						continue;
					}
					if(modules[parameterIndices[i][j]]==null) {
						done=false;
						continue outer;
					}
					parameters[j]=modules[parameterIndices[i][j]];
					if(parameters[j]==null){System.out.println("Module not available");}
				}
				try {
					modules[i]=(ProjectModule)factoryMethod.invoke(null,parameters);
				} catch(IllegalAccessException e) {
					e.printStackTrace();
				} catch(IllegalArgumentException e) {
					e.printStackTrace();
				} catch(InvocationTargetException e) {
					if(e.getCause() instanceof ParseException){throw (ParseException)e.getCause();}
					e.printStackTrace();
				}
			}
		}
		TestProject project=new TestProject();
		for(int i=0;i<modules.length;i++) {
			project.addModule(modules[i]);
		}
		return project;
	}

	static {
		BufferedReader reader=null;
		try {
			reader=new BufferedReader(new InputStreamReader(ClassLoader.getSystemResourceAsStream("modules.cfg")));
			String line;
			List classesList=new ArrayList();
			List constructorsList=new ArrayList();
			while(reader.ready()) {
				line=reader.readLine();
				if(line.equals("")){continue;}
				Class moduleClass=Class.forName(line);
				classesList.add(moduleClass);
				Method[] methods=moduleClass.getMethods();
				for(int i=0;i<methods.length;i++) {
					if(methods[i].getName().equals("createInstance")) {
						constructorsList.add(methods[i]);
						break;
					}
				}
			}
			classes=(Class[])classesList.toArray(new Class[0]);
			factoryMethods=(Method[])constructorsList.toArray(new Method[0]);
			parameterIndices=new int[classes.length][];
			Class[] parameterTypes;
			for(int i=0;i<factoryMethods.length;i++) {
				parameterTypes=factoryMethods[i].getParameterTypes();
				parameterIndices[i]=new int[parameterTypes.length];
				for(int j=0;j<parameterTypes.length;j++) {
					if(parameterTypes[j].equals(Document.class)){parameterIndices[i][j]=-1;continue;}
					for(int k=0;k<classes.length;k++) {
						if(parameterTypes[j].equals(classes[k])){parameterIndices[i][j]=k;break;}
					}
				}
			}
		} catch(FileNotFoundException e) {
			e.printStackTrace();
		} catch(IOException e) {
			e.printStackTrace();
		} catch(ClassNotFoundException e) {
			e.printStackTrace();
		} finally {
			try{if(reader!=null){reader.close();}}catch(IOException e){}
		}
	}
}
