package abchr;

import org.jdom.Document;
import org.jdom.Element;

import java.util.List;
import java.io.IOException;

public class RatingScaleModule extends AbstractProjectModule implements ConfigModule {
	private static final String[] DEFAULT_RATING_SCALE=new String[]{"Very annoying","Annoying","Slightly annoying","Perceptible, but not annoying","Imperceptible"};

	private String[] ratingScale;

	public RatingScaleModule(String[] ratingScale) {
		if(ratingScale.length!=5){throw new IllegalArgumentException("Rating scale must have exactly 5 labels");}
		this.ratingScale=ratingScale;
	}

	public RatingScaleModule() {
		this(new String[5]);
	}	

	public String getRatingScaleLabel(int index) {
		if(index<0 || index>4){throw new IllegalArgumentException();}
		return ratingScale[index]==null?DEFAULT_RATING_SCALE[index]:ratingScale[index];
	}

	public void setRatingScaleLabel(int index,String label) {
		if(index<0 || index>4){throw new IllegalArgumentException();}
		ratingScale[index]=label;
	}
	public String[] getRatingScale() {
		String[] result=new String[5];
		for(int i=0;i<5;i++){result[i]=ratingScale[i];}
		return result;
	}

	public static ProjectModule createInstance(Document doc) {
		Element root=doc.getRootElement().getChild("ratingscale");
		if(root==null){return new RatingScaleModule(new String[5]);}
		List labelList=root.getChildren("label");
		String[] labels=new String[5];
		for(int i=1;i<=5;i++) {
			labels[5-i]=((Element)labelList.get(i-1)).getTextTrim();
		}
		return new RatingScaleModule(labels);
	}

	public org.jdom.Element toXML() {
		out:{for(int i=0;i<5;i++){if(ratingScale[i]!=null){break out;}}return null;}
		org.jdom.Element root=new org.jdom.Element("ratingscale");
		org.jdom.Element label;
		for(int i=1;i<=5;i++) {
			label=new org.jdom.Element("label");
			label.addContent(new org.jdom.Text(ratingScale[5-i]));
			root.addContent(label);
		}
		return root;
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		//System.out.println("Writing RatingScaleModule");
		out.defaultWriteObject();
		//System.out.println("Done writing RatingScaleModule");
	}
}