package abchr;

import abchr.gui.SampleConfig;
import org.jdom.Document;
import org.jdom.Element;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.security.NoSuchAlgorithmException;

public class RawTestConfig extends AbstractProjectModule implements ConfigModule {
	private static File relativePathOutput;
	private static File relativePathInput;

	private SampleConfig original=new SampleConfig();
	private List<SampleConfig> samples=new ArrayList<SampleConfig>(8);

	private boolean randomizeSamples=false;

	private boolean calculateHashes=false;

	public SampleConfig getOriginal(){return original;}
	public void setOriginal(SampleConfig original){this.original=original;}

	public int getSampleCount(){return samples.size();}
	public SampleConfig get(int index){return samples.get(index);}
	public void add(SampleConfig sampleConfig){samples.add(sampleConfig);}
	public void remove(SampleConfig sampleConfig){samples.remove(sampleConfig);}
	public void remove(int index){samples.remove(index);}
	public void clear(){samples.clear();}

	public boolean getRandomizeSamples(){
		//System.out.println ("getRandomizeSamples: " + randomizeSamples);
		return randomizeSamples;
	}
	
	public void setRandomizeSamples(boolean randomizeSamples){
		//System.out.println ("setRandomizeSamples: " + randomizeSamples);
		this.randomizeSamples=randomizeSamples;
	}

	private static String convertToRelative(File file,File reference) throws IOException {
		File ref=reference.getCanonicalFile();
		File f=file.getCanonicalFile();
		boolean success=false;
		while(ref!=null) {
			f=file.getParentFile();
			while(f!=null) {
				if(f.equals(ref)){success=true;break;}
				f=f.getParentFile();
			}
			if(success){break;}
			ref=ref.getParentFile();
		}
		if(!success){return file.getCanonicalPath();}
		StringBuffer path=new StringBuffer();
		while(!reference.equals(ref)) {
			path.append("..");
			path.append(File.separatorChar);
			reference=reference.getParentFile();
		}
		StringBuffer p=new StringBuffer();
		while(!file.equals(f)) {
			p.insert(0,file.getName());
			p.insert(0,File.separatorChar);
			file=file.getParentFile();
		}
		p.deleteCharAt(0);
		path.append(p);
		return path.toString();
	}

	public void convertLocationsToRelative(File reference) throws IOException {
		String f="";
		try {
			original.setFilename(convertToRelative(new File(f=original.getFilename()),reference));
			SampleConfig sc;
			for(int i=0;i<samples.size();i++) {
				sc=get(i);
				sc.setFilename(convertToRelative(new File(f=get(i).getFilename()),reference));
				sc.setReference(reference);
			}
		} catch(IOException e) {
			throw new IOException(f+" is not a valid filename");
		}
	}

	public void convertLocationsToAbsolute() throws IOException {
		String f="";
		try {
			original.setFilename(new File(f=original.getFilename()).getCanonicalPath());
			for(int i=0;i<samples.size();i++) {
				get(i).absolutizeFilename();
			}
		} catch(IOException e) {
			throw new IOException(f+" is not a valid filename");
		}
	}

	public org.jdom.Element toXML() {
		org.jdom.Element root=new org.jdom.Element("testconfig");
		if(!randomizeSamples){root.setAttribute("randomize","false");}
		if(calculateHashes) {
			try {
				SampleHash.setHash(original,SampleHash.computeHash(original.getAbsoluteFile(),"SHA"));
				SampleConfig config;
				for(int i=0;i<samples.size();i++) {
					config=get(i);
					SampleHash.setHash(config,SampleHash.computeHash(config.getAbsoluteFile(),"SHA"));
				}
			} catch(IOException e) {
				e.printStackTrace();
			} catch(NoSuchAlgorithmException e) {
				e.printStackTrace();
			}
		}
		original.setReference(relativePathOutput);
		org.jdom.Element el=original.toXML();
		el.setName("original");
		root.addContent(el);
		SampleConfig sampleConfig;
		for(int i=0,n=samples.size();i<n;i++) {
			sampleConfig=get(i);
			sampleConfig.setReference(relativePathOutput);
			el=get(i).toXML();
			el.setName("testsample");
			root.addContent(el);
		}
		return root;
	}

	public static RawTestConfig loadFromXML(Document doc) throws ParseException {
		RawTestConfig result=new RawTestConfig();
		Element configElement=doc.getRootElement().getChild("testconfig");
		result.setRandomizeSamples("true".equals(configElement.getAttributeValue("randomize")));
		Element sample=configElement.getChild("original");
		SampleConfig original=SampleConfig.readFromXML(sample,relativePathInput);
		result.setOriginal(original);
		List samples=configElement.getChildren("testsample");
		for(int i=0,n=samples.size();i<n;i++) {
			sample=(Element)samples.get(i);
			SampleConfig sampleConfig=SampleConfig.readFromXML(sample,relativePathInput);
			result.add(sampleConfig);
		}
		return result;
	}

	public static ProjectModule createInstance(Document doc) throws ParseException {
		return loadFromXML(doc);
	}

	public static void setRelativePathOutput(File relativePathOutput) {
		RawTestConfig.relativePathOutput=relativePathOutput;
	}

	public static void setRelativePathInput(File relativePathInput) {
		RawTestConfig.relativePathInput=relativePathInput;
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		//System.out.println("Writing RawTestConfig");
		out.defaultWriteObject();
		//System.out.println("Done writing RawTestConfig");
	}

	public void setVerification(boolean b) {
		calculateHashes=b;
	}
}