package abchr;

import abchr.audio.Sample;
import abchr.gui.SampleConfig;

import java.util.Random;
import java.io.IOException;

import org.jdom.Element;

public class SampleConfigModule extends AbstractProjectModule {
	private static final Random rand=new MersenneTwister();

	private RawTestConfig rawConfig;
	private TestConfig config;
	private int[] indexMap;

	public SampleConfigModule(RawTestConfig rawConfig,TestConfig config) {
		this.rawConfig=rawConfig;
		this.config=config;
		indexMap=new int[config.getSampleCount()];
		int numSamples=config.getSampleCount();
		indexMap=new int[numSamples];
		for(int i=0;i<numSamples;i++){indexMap[i]=i;}
		if(rawConfig.getRandomizeSamples()) {
			int a,b;
			int temp;
			for(int i=0;i<2*numSamples;i++) {
				a=i<numSamples?i:rand.nextInt(numSamples);
				b=rand.nextInt(numSamples);
				temp=indexMap[a];
				indexMap[a]=indexMap[b];
				indexMap[b]=temp;
			}
		}
	}

	public Sample getSample(int index){return config.getSample(indexMap[index]);}
	public SampleConfig getSampleConfig(int index){return rawConfig.get(indexMap[index]);}

	public Sample getOriginalSample(){return config.getOriginalSample();}

	public int getSampleCount(){return config.getSampleCount();}

	public Element toResultsXML() {
		Element root=new Element("TestConfig");
		Element e=new Element("Reference");
		e.addContent(rawConfig.getOriginal().toXML().setName("Config"));
		root.addContent(e);
		for(int i=0,n=getSampleCount();i<n;i++) {
			e=new Element("Sample");
			e.addContent(new Element("ID").setText(Integer.toString(i)));
			e.addContent(getSampleConfig(i).getXML().setName("Config"));
			root.addContent(e);
		}
		return root;
	}

	public int indexOf(Sample sample) {
		if(sample==config.getOriginalSample()){return -1;}
		int realIndex=-1;
		for(int i=0,n=config.getSampleCount();i<n;i++) {
			if(config.getSample(i)==sample){realIndex=i;break;}
		}
		for(int i=0;i<indexMap.length;i++) {
			if(indexMap[i]==realIndex){return i;}
		}
		return -2;
	}

	public static ProjectModule createInstance(RawTestConfig rawConfig,TestConfig testConfig) {
		return new SampleConfigModule(rawConfig,testConfig);
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		//System.out.println("Writing SampleConfigModule");
		out.defaultWriteObject();
		//System.out.println("Done writing SampleConfigModule");
	}
}
