package abchr;

import abchr.crypto.BlowfishCipher;
import abchr.crypto.Cipher;
import abchr.crypto.Util;
import org.jdom.Document;

import java.io.*;
import java.util.Random;

public class ScrambledConfigIO implements ConfigIO {
	private static final int KEY_LENGTH=16;
	private static Random random=new MersenneTwister();

	private ConfigIO baseIO;
	private static final int MAX_LENGTH=50*1024;

	public Document readConfig(InputStream in) throws IOException {
		byte[] keyBytes=new byte[KEY_LENGTH];
		in.read(keyBytes);
		int[] key=Util.bytesToInts(keyBytes);
		Cipher blowfishCipher=new BlowfishCipher(key);
		byte[] lengthBytes=new byte[4];
		in.read(lengthBytes);
		int length=Util.byteArrayToInt(lengthBytes,0);
		if(length>MAX_LENGTH){throw new IOException("Config file has invalid format.");}
		byte[] cipherText=new byte[length];
		int read=0;
		while(read<length) {
			read+=in.read(cipherText,read,length-read);
		}
		byte[] text=blowfishCipher.decrypt(cipherText);
		ByteArrayInputStream stream=new ByteArrayInputStream(text);
		return baseIO.readConfig(stream);
	}

	public void writeConfig(OutputStream out,Document doc) throws IOException {
		ByteArrayOutputStream stream=new ByteArrayOutputStream();
		BufferedOutputStream bufferedStream=new BufferedOutputStream(stream);
		baseIO.writeConfig(bufferedStream,doc);
		bufferedStream.flush();
		bufferedStream.close();
		int[] key=new int[KEY_LENGTH/4];
		for(int i=0;i<KEY_LENGTH/4;i++){key[i]=random.nextInt();}
		Cipher blowfishCipher=new BlowfishCipher(key);
		out.write(Util.intsToBytes(key));
		byte[] cipherText=blowfishCipher.encrypt(stream.toByteArray());
		byte[] length=new byte[4];
		Util.intToByteArray(cipherText.length,length,0);
		out.write(length);
		out.write(cipherText);
	}

	public ScrambledConfigIO(ConfigIO baseIO){this.baseIO=baseIO;}
	public ScrambledConfigIO(){this(new XMLConfigIO());}
}
