package abchr;

import org.jdom.Document;

import java.io.*;

import abchr.crypto.*;

public class ScrambledResultIO implements ResultIO {
	private ResultIO baseIO=new XMLResultIO();
	private ElGamalPrivateKey key;

	public void setKey(ElGamalPrivateKey key) {
		this.key=key;
	}

	public void readKey(File f) throws IOException {
		ObjectInputStream keyInput=new ObjectInputStream(new FileInputStream(f));
		try {
			key=(ElGamalPrivateKey)keyInput.readObject();
		} catch(ClassNotFoundException e) {
			throw new IOException("Invalid key file.");
		}
		keyInput.close();
	}

	public Document readConfig(InputStream in) throws IOException {
		if(key==null){throw new IllegalStateException("Key not set in ScrambledResultIO.");}
		Decryptor decryptor=new ElGamalDecryptor(key);
		byte[] length=new byte[4];
		in.read(length);
		//System.out.println("length="+Util.byteArrayToInt(length,0));
		byte[] encryptedKey=new byte[Util.byteArrayToInt(length,0)];
		in.read(encryptedKey);
		byte[] blowfishKey;
		blowfishKey=decryptor.decrypt(encryptedKey);
		Cipher cipher=new BlowfishCipher(Util.bytesToInts(blowfishKey));
		in.read(length);
		//System.out.println("length="+Util.byteArrayToInt(length,0));
		byte[] encryptedText=new byte[Util.byteArrayToInt(length,0)];
		in.read(encryptedText);
		in.close();
		byte[] plainText=cipher.decrypt(encryptedText);
		return baseIO.readConfig(new ByteArrayInputStream(plainText));
	}

	public void writeConfig(OutputStream out,Document doc) throws IOException {
		ElGamalPublicKey key=((EncryptionKeyModule)EncryptionKeyModule.createInstance(doc)).getEncryptionKey();
		ByteArrayOutputStream bos=new ByteArrayOutputStream();
		baseIO.writeConfig(bos,doc);
		byte[] output=bos.toByteArray();
		if(key==null) {
			out.write(output);
		} else {
			//ByteArrayOutputStream output=new ByteArrayOutputStream(72+outputString.length()*8);
			Encryptor encryptor=new ElGamalEncryptor(key);
			byte[] blowfishKey=new byte[56];
			ElGamalEncryptor.rand.nextBytes(blowfishKey);
			//System.out.println();
			byte[] encryptedKey=encryptor.encrypt(blowfishKey);
			byte[] length=new byte[4];
			Util.intToByteArray(encryptedKey.length,length,0);
			out.write(length);
			out.write(encryptedKey);
			Cipher cipher=new BlowfishCipher(Util.bytesToInts(blowfishKey));
			output=cipher.encrypt(output);
			Util.intToByteArray(output.length,length,0);
			out.write(length);
			out.write(output);
		}
	}
}
