package abchr;

import abchr.audio.FileSample;
import abchr.audio.Sample;
import abchr.audio.SampleOffset;
import abchr.audio.SampleFactory;
import abchr.gui.SampleConfig;

import javax.sound.sampled.UnsupportedAudioFileException;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;

public class TestConfig extends AbstractProjectModule {
	private Sample originalSample;
	private final List<Sample> testSamples=new ArrayList<Sample>(8);

	private transient List resourceThief=new ArrayList(8);

	public Sample getOriginalSample(){return originalSample;}

	private void setOriginal(Sample original) {
		if(original instanceof FileSample) {
			try {
				InputStream stream=new FileInputStream(((FileSample)original).getWave());
				stream.read();
				resourceThief.add(stream);
			} catch(IOException e) {
				//e.printStackTrace();
			}
		}
		this.originalSample=original;
	}

	private void addSample(Sample sample) {
		if(sample instanceof FileSample) {
			try {
				InputStream stream=new FileInputStream(((FileSample)sample).getWave());
				stream.read();
				resourceThief.add(stream);
			} catch(IOException e) {
				//e.printStackTrace();
			}
		}
		testSamples.add(sample);
	}

	public int getSampleCount(){return testSamples.size();}
	public Sample getSample(int index){return testSamples.get(index);}

	public static ProjectModule createInstance(RawTestConfig rawConfig) throws ParseException {
		TestConfig result=new TestConfig();
		String file="";
		float minLength=Float.MAX_VALUE,length;
		try {
			file=rawConfig.getOriginal().getFilename();
			Sample sample=SampleFactory.createSample(rawConfig.getOriginal());
			length=sample.getLength()-SampleOffset.getOffset(rawConfig.getOriginal())/1000.0f;
			if(length<minLength){minLength=length;}
			result.setOriginal(sample);
			SampleOffset.setOffset(result.getOriginalSample(),SampleOffset.getOffset(rawConfig.getOriginal()));
			SampleConfig sc;
			for(int i=0,n=rawConfig.getSampleCount();i<n;i++) {
				sc=rawConfig.get(i);
				file=sc.getFilename();
				sample=SampleFactory.createSample(sc);
				length=sample.getLength()-SampleOffset.getOffset(rawConfig.get(i))/1000.0f;
				if(length<minLength){minLength=length;}
				result.addSample(sample);
			}
		} catch(UnsupportedAudioFileException e) {
			throw new ParseException(e.getMessage());
		} catch(IOException e) {
			throw new ParseException("File "+file+" could not be opened.");
		}
		result.getOriginalSample().setLength(minLength+SampleOffset.getOffset(rawConfig.getOriginal())/1000.0f);
		for(int i=0,n=rawConfig.getSampleCount();i<n;i++) {
			result.getSample(i).setLength(minLength+SampleOffset.getOffset(rawConfig.get(i))/1000.0f);
		}
		return result;
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		//System.out.println("Writing TestConfig");
		out.defaultWriteObject();
		//System.out.println("Done writing TestConfig");
	}

}