package abchr;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Attribute;

import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.*;
import java.text.*;
import java.text.ParseException;

import abchr.gui.SampleConfig;
import abchr.audio.SampleProcessor;
import abchr.audio.SampleProcessing;
import abchr.audio.Sample;
import abchr.audio.AbstractSample;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;

public class TxtResultIO implements ResultIO {
	public Document readConfig(InputStream in) throws IOException {
		// not implemented right now
		return null;
	}

	public void writeConfig(OutputStream out,Document doc) throws IOException {
		Element root=doc.getRootElement();
		StringBuffer buffer=new StringBuffer();
		final String nl=System.getProperty("line.separator");
		buffer.append(root.getChild("Version").getTextTrim()).append(", ").append(root.getChild("Date").getTextTrim());
		buffer.append(nl);
		buffer.append("Testname: ").append(root.getChild("TestName").getTextTrim()).append(nl).append(nl);
		Element e;
		if((e=root.getChild("TesterName"))!=null) {
			buffer.append("Tester: ").append(e.getTextTrim()).append(nl).append(nl);
		}
		String origName;
		SampleConfig[] sampleConfigs;
		e=root.getChild("TestConfig");
		Attribute rand=e.getAttribute("randomize");
		if(rand!=null && rand.getValue().equals("false")) {
			buffer.append("WARNING! SAMPLES WERE NOT RANDOMIZED!").append(nl);
		}
		try {
			final Element r=e.getChild("Reference");
			if(r.getChild("Config")!=null) {
				origName=makeName(SampleConfig.readFromXML(r.getChild("Config"),null));
			} else {
				origName=e.getChildTextTrim("Filename");
			}
		} catch(abchr.ParseException e1) {
			throw new IOException(e1.getMessage());
		}
		java.util.List sampleList=e.getChildren("Sample");
		sampleConfigs=new SampleConfig[sampleList.size()];
		String[] sampleNames=new String[sampleList.size()];
		int id;
		for(int i=0;i<sampleList.size();i++) {
			e=(Element)sampleList.get(i);
			id=Integer.parseInt(e.getChild("ID").getTextTrim());
			try {
				if(e.getChild("Config")!=null) {
					sampleConfigs[id]=SampleConfig.readFromXML(e.getChild("Config"),null);
				} else {
					sampleConfigs[id]=new SampleConfig(e.getChildTextTrim("Filename"),null);
				}
			} catch(abchr.ParseException e1) {
				throw new IOException(e1.getMessage());
			}
			sampleNames[id]=makeName(sampleConfigs[id]);
		}
		List comparisons=root.getChild("ABCHR").getChildren("Comparison");
		int refIndex;
		for(int i=0;i<comparisons.size();i++) {
			e=(Element)comparisons.get(i);
			id=Integer.parseInt(e.getChild("SampleID").getTextTrim());
			refIndex=Integer.parseInt(e.getChild("RefIndex").getTextTrim());
			buffer.append(id+1).append(refIndex==0?"R":"L").append(" = ").append(sampleNames[id]).append(nl);
		}
		buffer.append(nl).append("---------------------------------------").append(nl);
		buffer.append("General Comments: ").append(root.getChildText("Comments")).append(nl);
		buffer.append("---------------------------------------").append(nl);
		int ratedIndex;
		for(int i=0;i<comparisons.size();i++) {
			e=(Element)comparisons.get(i);
			id=Integer.parseInt(e.getChild("SampleID").getTextTrim());
			refIndex=Integer.parseInt(e.getChild("RefIndex").getTextTrim());
			ratedIndex=Integer.parseInt(e.getChild("RankedIndex").getTextTrim());
			if(ratedIndex==-1){continue;}
			String prefix=(id+1)+(ratedIndex==0?"L ":"R ");
			buffer.append(prefix).append("File: ").append(ratedIndex==refIndex?origName:sampleNames[id]).append(nl);
			buffer.append(prefix).append("Rating: ").append(e.getChildTextTrim("Rating")).append(nl);
			buffer.append(prefix).append("Comment: ").append(e.getChildTextTrim("Comment")).append(nl);
			buffer.append("---------------------------------------").append(nl);
		}

		buffer.append(nl).append("ABX Results:").append(nl);
		SimpleABXTestFormatter simpleFormatter=new SimpleABXTestFormatter(null);
		simpleFormatter.setFormattingString("$A_SAMPLE$ vs $B_SAMPLE$"+nl+"    $FINAL_RESULT$"+nl);
		simpleFormatter.setTrialFormattingString("$CORRECT$ out of $TOTAL$, pval $PVAL_EXT$");
		class SampleDummy extends AbstractSample {
			public AudioFormat getFormat(){return null;}
			public float getLength(){return 0;}
			public AudioInputStream getStream(){return null;}
			public void setLength(float length){}
			public SampleDummy(String name){setAttribute("name",name);}
		}
		Sample[] samples=new Sample[sampleList.size()+1];
		samples[0]=new SampleDummy("Original");
		for(int i=0;i<samples.length-1;i++) {
			samples[i+1]=new SampleDummy(sampleNames[i]);
		}
		boolean empty=true;
		comparisons=root.getChild("ABX").getChildren("Comparison");
		List<ABXTest> tests=new ArrayList<ABXTest>();
		ABXTest test;
		Element el;
		for(Object o : comparisons) {
			el=(Element)o;
			try {
				test=readTestFromXML(el,samples[Integer.parseInt(el.getChildTextTrim("SampleA"))+1],samples[Integer.parseInt(el.getChildTextTrim("SampleB"))+1]);
			} catch(JDOMException e1) {
				throw new IOException(e1.getMessage());
			}
			if(test.getNumTrials()==0){continue;}
			empty=false;
			tests.add(test);
		}
		for(ABXTest t : tests) {
			simpleFormatter.setTest(t);
			buffer.append(simpleFormatter.format());
		}
		if(!empty) {
			buffer.append(nl);
			buffer.append(nl);
			buffer.append("---- Detailed ABX results ----");
			buffer.append(nl);
		}
		DetailedABXTestFormatter detailedFormatter=new DetailedABXTestFormatter(null);
		detailedFormatter.setFormattingString("$A_SAMPLE$ vs $B_SAMPLE$"+nl+"$RESULT_LIST$");
		detailedFormatter.setTrialFormattingString("$RANGE_COND$    $TIME$ $PASSFAIL_S$ $CORRECT$/$TOTAL$ pval $PVAL_EXT$");
		detailedFormatter.setRangeFormatter(new DetailedABXTestFormatter.SimpleRangeFormatter() {
			public String format(int rangeBegin,int rangeEnd) {
				return "Playback Range: "+formatTime(rangeBegin)+" to "+formatTime(rangeEnd)+nl;
			}
		});
		for(ABXTest t : tests) {
			if(t.getNumTrials()==0){continue;}
			detailedFormatter.setTest(t);
			buffer.append(detailedFormatter.format());
			buffer.append(nl);
		}

		/*buffer.append(nl).append(nl).append("---- Hashes ----").append(nl);
		e=root.getChild("OS");
		if(e!=null) {
			buffer.append(nl).append("OS: ").append(e.getChildTextTrim("Architecture"));
			buffer.append(", ").append(e.getChildTextTrim("Name")).append(" ").append(e.getChildTextTrim("Version")).append(nl);
		}
		e=root.getChild("TestConfig").getChild("Reference");
		buffer.append("Original: ").append(e.getChildTextTrim("HashMatch")).append(nl);
		for(int i=0;i<sampleList.size();i++) {
			e=(Element)sampleList.get(i);
			buffer.append("Sample").append(Integer.parseInt(e.getChildTextTrim("ID"))+1).append(": ").append(e.getChildTextTrim("HashMatch")).append(nl);
		}*/
		out.write(buffer.toString().getBytes());
	}

	private String makeName(SampleConfig config) {
		StringBuffer b=new StringBuffer();
		b.append(config.getFilename());
		List<SampleProcessor> queue=SampleProcessing.getProcessingQueue(config);
		if(queue!=null && !queue.isEmpty()) {
			SampleProcessor p;
			for(Iterator<SampleProcessor> it=queue.iterator();it.hasNext();) {
				p=it.next();
				b.append(" | ");
				b.append(p.toString());
			}
		}
		return b.toString();
	}

	private static ABXTest readTestFromXML(Element e,Sample aSample,Sample bSample) throws JDOMException {
		DateFormat dateFormat=new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ",Locale.getDefault());
		DateFormat rangeFormat=new SimpleDateFormat("mm:ss.SSS z");
		ABXTest test=new ABXTestImpl(aSample,bSample);
		Element progress=e.getChild("Progress");
		List trials=progress.getChildren("Trial");
		Element trial;
		for(Object o : trials) {
			trial=(Element)o;
			try {
				String start=trial.getChildTextTrim("RangeStart");
				if(start.indexOf(':')==-1) {
					start="00:"+start;
				}
				String end=trial.getChildTextTrim("RangeEnd");
				if(end.indexOf(':')==-1) {
					end="00:"+end;
				}
				start=start+" GMT";
				end=end+" GMT";
				test.addTrial(new ABXTrial(
					trial.getChildTextTrim("Correct").equals("true"),
					dateFormat.parse(trial.getChildTextTrim("Time")).getTime(),
					(int)rangeFormat.parse(start).getTime(),
					(int)rangeFormat.parse(end).getTime()
				));
			} catch(ParseException e1) {
				throw new JDOMException(e1.getMessage());
			}
		}
		return test;
	}
}
