package abchr.audio;

import guiutils.PositionMarker;

import javax.sound.sampled.*;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.io.IOException;
import java.util.Vector;
import java.util.Arrays;


public class PlaybackThread implements ChangeListener {
	private static final PlaybackThread INSTANCE=new PlaybackThread();

	public static PlaybackThread getInstance() {			
		return INSTANCE;
	}

	private Mixer mixer;

	private int bufferSize;

	private Player currentPlayer;
	private float start=0.0f;
	private float end=-1.0f;
	private float startedAt=0.0f;
	private boolean fastSwitching=true;
	private boolean loop;
	private BoundedRangeModel rangeModel;
	private PositionMarker posMarker=new PositionMarker();

	private boolean streams_skiped = false;
	
	private Thread t;

	private class Player implements Runnable {
		private Vector<Sample> samples;
		private Vector<AudioInputStream> streams;
		private Vector<Vector<Sample>> all_samples;
		private Vector<Vector<AudioInputStream>> all_streams;
		
		private int streamIndex;
		private int curStreamsStart;  
		private int curStreamsEnd;
	
		// curStream should probably be synchronized, but only one thing writes to it
		private int curStream;

		private SourceDataLine line;
		private byte[] buffer;

		private AudioFormat format;
		private float format_rate;
		private int format_size;
		private int line_avail;

		private float prevGain;
		private float curGain = 0;
		
		private FloatControl gainControl; 
		
			
		public Player() {
			//System.out.println("end="+end);
		}
		
		
		public void newSession() {
			// This shouldn't leak memory...
			//System.out.println("newSession:");
			if (all_streams != null) {
				for (int i = 0; i<all_streams.size(); i++){
					streams=all_streams.get(i);
					for (int j = 0; j<streams.size(); j++){
						//System.out.println("newSession clossing " + i + " " + j);
						try {streams.get(j).close();}catch (Exception e) {e.printStackTrace();}
					}
				}
			}
			all_samples = new Vector<Vector<Sample>>();
			all_streams = new Vector<Vector<AudioInputStream>>();

			streamIndex = -1;
			curStreamsStart = -1;  
			curStreamsEnd = -1;
			
			
		}



		public void newSamples(int nSamples) {
			//System.out.println ("newSamples: " + nSamples);
			samples = new Vector<Sample>(nSamples);
			all_samples.add(samples);

			streams = new Vector<AudioInputStream>(nSamples);
			all_streams.add(streams);

		}


		public int addSample (Sample s) {

			samples.add(s);
			streams.add(s.getStream());
			
			streamIndex++;
			//streams_skiped = false;

			//System.out.println ("addSample: "+ (streams.size()-1));

			//System.out.println ("addSample returning : "+ streamIndex);
			return streamIndex;
			
		}

		public void openLine (Sample sample) {
			format=sample.getFormat();
			//System.out.println("openLine Format: " + format.toString());
			format_rate=format.getFrameRate();
			format_size=format.getFrameSize();
			
			DataLine.Info info=new DataLine.Info(SourceDataLine.class,format,format.getFrameSize()*(int)(bufferSize*format.getSampleRate()/1000));
			try {
				if (line==null) {
					line=(SourceDataLine)mixer.getLine(info);
				}
				//System.out.println("Opening line");
				line.open(format,format.getFrameSize()*(int)(bufferSize*format.getSampleRate()/1000));
			} catch(LineUnavailableException e) {
				e.printStackTrace();
				System.out.println("\nLine unavailable, probably in use.\n");
				return;
			} catch(IllegalArgumentException e) {
				System.out.println("Mixer info: " + mixer.getMixerInfo().toString());
				System.out.println("Mixer unavailable, probably in use or try choosing another one\n");
				line=null;
				return;
			} catch(Exception e){
				e.printStackTrace();
				line=null;
				return;
			}
			gainControl=(FloatControl)line.getControl(FloatControl.Type.MASTER_GAIN);
			buffer=new byte[line.getBufferSize()];
		}

		private void startLine () {
			//System.out.println("startLine: ");
			if (line==null || !line.isOpen()) {
				openLine(samples.get(0));
			}
			line.start();
			//System.out.println("startLine: line open =  "+line.isOpen());
			//System.out.println("startLine: line active =  "+line.isActive());
			//System.out.println("startLine: line running =  "+line.isRunning());
		}
		
		private void initStreamSkips() {
			//System.out.println("initStreamSkips: " + start);
			try {
				for (int i = 0; i<samples.size(); i++){
					int offset=SampleOffset.getOffset(samples.get(i));
					int skip=((int)((start+offset/1000.0f)*format_rate))*format_size;
					//System.out.println("initStreamSkips: start = " + start + ", offset = " + offset + ", format_rate = " + format_rate  + ", format_size = " + format_size);
					//System.out.println("initStreamSkips: skipping " + skip + ", i = " + i);
					(streams.get(i)).skip(skip);
				}
				
			} catch(IOException e) {
				e.printStackTrace();
			}
			//System.out.println("initStream Skips: setting streams_skiped to true");
			streams_skiped=true;
		}

		private void resetStreams() {
			//System.out.println("resetStreams: ");
			try {
				for (int i = 0; i<samples.size(); i++){
					//System.out.println("resetStreams: " + i);
					streams.get(i).close();
					streams.set(i,samples.get(i).getStream());
				}
			} catch(Exception e) {
				e.printStackTrace();
			}
		}


		synchronized void setcurStream (int s) {
			//System.out.println("setcurStream: ");
			if (s<curStreamsStart || s>curStreamsEnd) {
				int i=-1;
				int k=-1;
				while (i < s) {
					k++;
					i+=all_streams.get(k).size();
				}
				samples = all_samples.get(k);
				streams = all_streams.get(k);
				
				curStreamsStart=i-(all_streams.get(k).size()-1);
				curStreamsEnd=i;
			}


			curStream = s-curStreamsStart;
			
			prevGain = curGain;
			curGain = SampleGain.getGain(samples.get(curStream));
			
			if (curGain != prevGain){
				gainControl.setValue(curGain);
			}
			
			try {mynotify();}   
			catch (InterruptedException e) {e.printStackTrace();}
			
		}
		
		synchronized void mywait () throws InterruptedException {
			wait();
		}
		
		synchronized void mynotify () throws InterruptedException {
			notify();
		}

		public void run() {
			int n=0;
			int wrote;
			long linePosition;
			while (true) {
				try {
					//System.out.println ("main waiting");
					mywait();
					//System.out.println ("main notified");
					while(line.isOpen()) {
						linePosition=line.getMicrosecondPosition();
						if(posMarker!=null) {
							//System.out.println("lineposition="+(int)(startedAt*1000+linePosition/1000l));
							posMarker.setPosition((int)(startedAt*1000+linePosition/1000l));
						}
						//System.out.println ("pos = " + (startedAt+linePosition/1000000.0f+0.01f) + ", end = " + end);
						if(/*end!=-1 && */startedAt+linePosition/1000000.0f+0.01f>end) {
							startedAt=start;
							stopPlayback();
							initStreamSkips();
							if(isLooping()) {
								startLine();
								try {mynotify();}   
								catch (InterruptedException e) {e.printStackTrace();}
								continue;
							} else {
								break;
							}
						}
						line_avail = line.available();
						//System.out.println("line. avail =  "+ line_avail);
						if(line_avail<8192){try{Thread.sleep(20);}catch(InterruptedException e){}continue;}
						
						n=streams.get(curStream).read(buffer,0,line_avail);
						if(n==-1) {try{Thread.sleep(20);}catch(InterruptedException e){}continue;}
					
						
						for (int i = 0 ; i<samples.size(); i++) {
							if (i!=curStream) {
								streams.get(i).skip((long)line_avail);
							}
						}
						
						wrote=line.write(buffer,0,n);
						if (wrote != n) {
							System.out.println("did not write " + n + " bytes to audio line, only " + wrote);
						}
					}
					
					
					if(n!=-1) {
						posMarker.setPosition(-1);
					}
				} catch(IOException e) {
					e.printStackTrace();
				} catch(InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
	
		public float getPosition() {
			return startedAt+line.getMicrosecondPosition()/1000000.0f;
		}

		public synchronized void stopPlayback() {
			//System.out.println("stopping2");
			if (line!=null) {
				line.stop();
				line.flush();
				line.close();
				resetStreams(); 
				startedAt=start;
				streams_skiped=false;
			}
			
		}
		
		public synchronized void shutdown() {
			stopPlayback();
		}


		
		public boolean lineNotNull() { return line!=null;}
		public boolean isPlaying() { return (line!=null && line.isOpen());}
		
	}

	private PlaybackThread() {
		Mixer.Info[] info=AudioSystem.getMixerInfo();
		/*System.out.println(Arrays.deepToString(info));
		for(Mixer.Info in : info) {
			System.out.println(in.getDescription());
			Mixer mixer=AudioSystem.getMixer(in);
			Line.Info[] sourceLineInfo=mixer.getSourceLineInfo();
			for(int i=0;i<sourceLineInfo.length;i++) {
				System.out.println(sourceLineInfo[i].toString());
			}
		}
		mixer=null;
		for(int i=0;i<info.length;i++) {
			if(info[i].getName().startsWith("Java Sound")) {
				mixer=AudioSystem.getMixer(info[i]);
				javaAudioMixer=true;
			}
		}*/
		for(Mixer.Info in : info) {
			Mixer m=AudioSystem.getMixer(in);
			if(m.getSourceLineInfo().length!=0){mixer=m;}
		}
		//mixer=AudioSystem.getMixer(info[0]);
		Control[] controls=mixer.getControls();
		for(int i=0;i<controls.length;i++) {
			if(controls[i].getType()==EnumControl.Type.REVERB) {
				Object[] values=((EnumControl)controls[i]).getValues();
				ReverbType rt;
				for(int j=0;j<values.length;j++) {
					rt=(ReverbType)values[j];
					if(rt.getEarlyReflectionIntensity()==0.0f && rt.getLateReflectionIntensity()==0.0f && rt.getDecayTime()==0) {
						((EnumControl)controls[i]).setValue(values[j]);
						//System.out.println("Set Reverb to "+controls[i]);
					}
				}
			}
		}
		currentPlayer = new Player();
		t=new Thread(currentPlayer);
		t.start();
	}
	
	public void shutdown() {
		currentPlayer.shutdown();
	}

	public void play(int s) {

		//System.out.println ("play: start = " + start + ", startedAt = " + startedAt);
		
		if (!fastSwitching) {
			startedAt=start;
			//System.out.println ("play: stopping()");
			currentPlayer.stopPlayback();
			
		}
		
		currentPlayer.startLine();

		if (!streams_skiped) {
			currentPlayer.initStreamSkips();
		}
		
		currentPlayer.setcurStream(s);
		Thread.yield();
	}


	public int addSample (Sample s) {
		return currentPlayer.addSample (s);
	}

	public void newSession () {
		currentPlayer.newSession ();
	}

	public void newSamples (int n) {
		currentPlayer.newSamples (n);
	}

	public void stopPlayback() {
		//System.out.println("stopping1");
		if(currentPlayer!=null){currentPlayer.stopPlayback();}
	}

	public boolean isFastSwitching(){return fastSwitching;}
	public void setFastSwitching(boolean fastSwitching){this.fastSwitching=fastSwitching;}
	public synchronized boolean isLooping(){return loop;}
	public synchronized void setLooping(boolean loop){this.loop=loop;}

	public void stateChanged(ChangeEvent e) {
		start=rangeModel.getValue()/1000.0f;
		end=(rangeModel.getValue()+rangeModel.getExtent())/1000.0f;

		//System.out.println("stateChanged:");

		if (!currentPlayer.isPlaying()){
			//System.out.println("stateChanged: not playing");
			streams_skiped=false;
			startedAt=start;
		}
		
		if (currentPlayer.lineNotNull() && start > currentPlayer.getPosition() && currentPlayer.isPlaying()){
			//System.out.println("stateChanged: start later than curr pos");
			startedAt=start;
			currentPlayer.stopPlayback();
			currentPlayer.initStreamSkips();
			currentPlayer.startLine();
			try {currentPlayer.mynotify();}   
			catch (InterruptedException x) {x.printStackTrace();}
		}
		Thread.yield();
	}

	public void setRangeModel(BoundedRangeModel model) {
		if(rangeModel!=null){rangeModel.removeChangeListener(this);}
		rangeModel=model;
		if(rangeModel!=null){rangeModel.addChangeListener(this);}
		stateChanged(null);
	}

	public BoundedRangeModel getRangeModel(){return rangeModel;}
	public PositionMarker getPosMarker(){return posMarker;}

	public int getBufferSize(){return bufferSize;}
	public void setBufferSize(int bufferSize){this.bufferSize=bufferSize; stopPlayback();}

	public Mixer.Info getMixer(){return mixer.getMixerInfo();}
	public void setMixer(Mixer.Info info){stopPlayback();mixer=AudioSystem.getMixer(info);}
}