package guiutils;

import java.awt.*;

public class FlexibleGridLayout implements LayoutManager {
	private int rows,cols,hgap=0,vgap=0;
	private int[] rowHeights;
	private int[] colWidths;

	private boolean fill=false;

	public void addLayoutComponent(String name,Component comp) {
	}

	public void removeLayoutComponent(Component comp) {
	}

	public void layoutContainer(Container parent) {
		calcSizes(parent);
		int n=parent.getComponentCount();
		Insets in=parent.getInsets();
		int x=in.left,y=in.top;
		int row,col;
		Component c;
		Dimension d;
		for(int i=0;i<n;i++) {
			row=i/cols;col=i%cols;
			c=parent.getComponent(i);
			d=c.getPreferredSize();
			if(fill) {
				c.setBounds(x,y,colWidths[col],rowHeights[row]);
			} else {
				c.setBounds(x+colWidths[col]/2-d.width/2,y+rowHeights[row]/2-d.height/2,d.width,d.height);
			}
			if(col==cols-1){x=in.left;y+=rowHeights[row]+vgap;}else{x+=colWidths[col]+hgap;}
		}
	}

	public Dimension minimumLayoutSize(Container parent) {
		return preferredLayoutSize(parent);
	}

	public Dimension preferredLayoutSize(Container parent) {
		calcSizes(parent);
		Insets in=parent.getInsets();
		int height=in.top+in.bottom+vgap*(rows-1);
		for(int i=0;i<rows;i++){height+=rowHeights[i];}
		int width=in.left+in.right+hgap*(cols-1);
		for(int i=0;i<cols;i++){width+=colWidths[i];}
		return new Dimension(width,height);
	}

	private void calcSizes(Container parent) {
		for(int i=0;i<rows;i++){rowHeights[i]=0;}
		for(int i=0;i<cols;i++){colWidths[i]=0;}
		int n=parent.getComponentCount();
		rows=n/cols+1;
		if(rowHeights.length<rows){rowHeights=new int[rows];}
		Component c;
		int row,col;
		Dimension d;
		for(int i=0;i<n;i++) {
			row=i/cols;col=i%cols;
			c=parent.getComponent(i);
			if(c.isVisible()) {
				d=c.getPreferredSize();
				if(d.height>rowHeights[row]){rowHeights[row]=d.height;}
				if(d.width>colWidths[col]){colWidths[col]=d.width;}
			}
		}
	}

	public FlexibleGridLayout(int rows,int cols){this(rows,cols,0,0);}
	public FlexibleGridLayout(int rows,int cols,int hgap,int vgap){this(rows,cols,hgap,vgap,false);}

	public FlexibleGridLayout(int rows,int cols,int hgap,int vgap,boolean fill) {
		this.cols=cols;
		this.hgap=hgap;
		//this.rows=rows;
		this.vgap=vgap;
		rowHeights=new int[rows];
		colWidths=new int[cols];
		this.fill=fill;
	}
}
