package guiutils;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class LineLayout implements LayoutManager {
	private static final class LineBreak extends JComponent{public String toString(){return "LINE_BREAK";}};
	public static Component lineBreak(){return new LineBreak();}

	public static final int LEFT_ALIGN=0;
	public static final int RIGHT_ALIGN=1;
	public static final int CENTER=2;
	public static final int TOP_ALIGN=3;
	public static final int BOTTOM_ALIGN=4;

	private List lines=new ArrayList(2);
	private int lineCount;
	private int[] lineHeights;
	private int[] lineWidths;
	private final int hAlign;
	private final int vAlign;

	private final int hGap;
	private final int vGap;

	private final boolean fillVertical;

	public void addLayoutComponent(String name,Component comp){}
	public void removeLayoutComponent(Component comp){}

	public void layoutContainer(Container parent) {
		update(parent);
		Insets in=parent.getInsets();
		int x,y;
		if(vAlign==TOP_ALIGN) {
			y=in.top;
		} else {
			int totalHeight=0;
			for(int h : lineHeights){totalHeight+=h+vGap;}
			totalHeight-=vGap;
			if(vAlign==BOTTOM_ALIGN) {
				y=in.bottom-totalHeight;
			} else {
				y=(parent.getHeight()-in.top-in.bottom-totalHeight)/2+in.top;
			}
		}
		Component c;
		Dimension d;
		List currentLine;
		for(int i=0;i<lineCount;i++) {
			x=hAlign==LEFT_ALIGN?in.left:(hAlign==RIGHT_ALIGN?(parent.getWidth()-in.right-lineWidths[i]):((parent.getSize().width-lineWidths[i])/2));
			currentLine=(List)lines.get(i);
			for(int j=0;j<currentLine.size();j++) {
				c=(Component)currentLine.get(j);
				d=c.getPreferredSize();
				c.setBounds(x,y,d.width,fillVertical?lineHeights[i]:d.height);
				x+=d.width+hGap;
			}
			y+=lineHeights[i]+vGap;
		}
	}

	public Dimension minimumLayoutSize(Container parent) {
		return preferredLayoutSize(parent);
	}

	public Dimension preferredLayoutSize(Container parent) {
		update(parent);
		Insets in=parent.getInsets();
		int height=0;
		for(int i=0;i<lineCount;i++){height+=lineHeights[i];}
		int width=0;
		for(int i=0;i<lineCount;i++){width=width>lineWidths[i]?width:lineWidths[i];}
		return new Dimension(width+in.left+in.right,height+in.top+in.bottom+vGap*(lines.size()-1));
	}

	private void update(Container parent) {
		int n=parent.getComponentCount();
		Component c;
		int newLineCount=1;
		for(int i=0;i<n;i++) {
			c=parent.getComponent(i);
			if(c instanceof LineBreak){++newLineCount;}
		}
		if(newLineCount!=lineCount) {
			lines.clear();
			List line=new ArrayList();
			for(int i=0;i<n;i++) {
				c=parent.getComponent(i);
				if(c instanceof LineBreak) {
					lines.add(line);
					line=new ArrayList();
				} else {
					line.add(c);
				}
			}
			lines.add(line);
			lineCount=lines.size();
			lineHeights=new int[lineCount];
			lineWidths=new int[lineCount];
		} else {
			int j=0;
			List currentLine=(List)lines.get(0);
			currentLine.clear();
			for(int i=0;i<n;i++) {
				c=parent.getComponent(i);
				if(c instanceof LineBreak) {
					(currentLine=(List)lines.get(++j)).clear();
				} else {
					currentLine.add(c);
				}
			}
		}
		int width,height;
		List currentLine;
		Dimension size;
		for(int i=0;i<lineCount;i++) {
			currentLine=(List)lines.get(i);
			width=0;
			height=0;
			for(int j=0;j<currentLine.size();j++) {
				size=((Component)currentLine.get(j)).getPreferredSize();
				width+=size.width+hGap;
				height=height>size.height?height:size.height;
			}
			width-=hGap;
			lineWidths[i]=width;
			lineHeights[i]=height;
		}
	}

	public LineLayout(int hGap,int vGap,int hAlign,int vAlign,boolean fillVertical) {
		this.hGap=hGap;
		this.vGap=vGap;
		this.hAlign=hAlign;
		this.vAlign=vAlign;
		this.fillVertical=fillVertical;
		lines.add(new ArrayList(2));
	}

	public LineLayout(int hGap,int vGap,int hAlign,boolean fillVertical){this(hGap,vGap,hAlign,TOP_ALIGN,fillVertical);}
	public LineLayout(){this(5,5,LEFT_ALIGN,TOP_ALIGN,false);}
	public LineLayout(int hGap,int vGap){this(hGap,vGap,LEFT_ALIGN,TOP_ALIGN,false);}
	public LineLayout(int alignment){this(5,5,alignment,TOP_ALIGN,false);}
	public LineLayout(int alignment,boolean fillVertical){this(5,5,alignment,TOP_ALIGN,fillVertical);}
	public LineLayout(int hGap,int vGap,int alignment){this(hGap,vGap,alignment,TOP_ALIGN,false);}
}