package guiutils;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.io.File;

public class SimpleFileChooser extends JFileChooser {
	public SimpleFileChooser(String currentDirectoryPath) {
		super(currentDirectoryPath);
	}

	public SimpleFileChooser(String currentDirectoryPath,FileFilter filter) {
		super(currentDirectoryPath);
		this.addChoosableFileFilter(filter);
	}

	public SimpleFileChooser(){this(System.getProperty("abchr.dir")!=null?System.getProperty("abchr.dir"):System.getProperty("user.dir"));}
	public SimpleFileChooser(FileFilter filter){this(System.getProperty("abchr.dir")!=null?System.getProperty("abchr.dir"):System.getProperty("user.dir"),filter);}

	public File openFile(Component parent) {
		if(showOpenDialog(parent)==JFileChooser.APPROVE_OPTION) {
			if(!this.getSelectedFile().exists()) {
				JOptionPane.showMessageDialog(parent,"The file you selected does not exist","Error",JOptionPane.ERROR_MESSAGE);
				return null;
			}
			return this.getSelectedFile();
		} else {
			return null;
		}
	}

	public int showOpenDialog(Component parent) throws HeadlessException {
		int r=super.showOpenDialog(parent);
		if(r==JFileChooser.APPROVE_OPTION) {
			System.setProperty("abchr.dir",this.getCurrentDirectory().getAbsolutePath());
		}
		return r;
	}

	public File saveFile(Component parent) {
		File f;
		int option=-1;
		do {
			if(this.showSaveDialog(parent)!=JFileChooser.APPROVE_OPTION){return null;}
			f=this.getSelectedFile();
			if(this.getFileFilter() instanceof SimpleFileFilter) {
				String extension=((SimpleFileFilter)this.getFileFilter()).getExtension();
				if(!f.getName().endsWith(extension)) {
					f=new File(f.getPath()+extension);
				}
			}
			if(f.exists()) {
				option=JOptionPane.showConfirmDialog(this,"The file you selected already exists.\nOverwrite?","Overwrite?",JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE);
				if(option==JOptionPane.CANCEL_OPTION){return null;}
			} else{break;}
		} while(option!=JOptionPane.YES_OPTION);
		System.setProperty("abchr.dir",this.getCurrentDirectory().getAbsolutePath());
		return f;
	}

	private static SimpleFileChooser STD_FILE_CHOOSER;

	public static File openFile(Component parent,FileFilter[] filters,boolean allFileFilterUsed) {
		init();
		STD_FILE_CHOOSER.setAcceptAllFileFilterUsed(allFileFilterUsed);
		for(int i=0;i<filters.length;i++){STD_FILE_CHOOSER.addChoosableFileFilter(filters[i]);}
		STD_FILE_CHOOSER.setFileFilter(filters[0]);
		File file=STD_FILE_CHOOSER.openFile(parent);
		STD_FILE_CHOOSER.resetChoosableFileFilters();
		STD_FILE_CHOOSER.setAcceptAllFileFilterUsed(true);
		return file;
	}

	public static File defaultOpenFile(Component parent) {
		init();
		return STD_FILE_CHOOSER.openFile(parent);
	}

	public static File openFile(Component parent,FileFilter filter) {
		return openFile(parent,new FileFilter[]{filter},true);
	}

	public static File openFile(Component parent,FileFilter filter,boolean allFileFilterUsed) {
		return openFile(parent,new FileFilter[]{filter},allFileFilterUsed);
	}

	public static File saveFile(Component parent,FileFilter[] filters,boolean allFileFilterUsed) {
		init();
		for(int i=0;i<filters.length;i++){STD_FILE_CHOOSER.addChoosableFileFilter(filters[i]);}
		STD_FILE_CHOOSER.setAcceptAllFileFilterUsed(allFileFilterUsed);
		if(filters.length>0){STD_FILE_CHOOSER.setFileFilter(filters[0]);}
		STD_FILE_CHOOSER.setSelectedFile(new File(""));
		File file=STD_FILE_CHOOSER.saveFile(parent);
		for(int i=0;i<filters.length;i++){STD_FILE_CHOOSER.removeChoosableFileFilter(filters[i]);}
		STD_FILE_CHOOSER.setAcceptAllFileFilterUsed(true);
		return file;
	}

	public static File defaultSaveFile(Component parent) {
		init();
		return STD_FILE_CHOOSER.saveFile(parent);
	}

	public static File saveFile(Component parent,FileFilter filter) {
		return saveFile(parent,new FileFilter[]{filter},true);
	}

	public static File saveFile(Component parent,FileFilter filter,boolean allFileFilterUsed) {
		return saveFile(parent,new FileFilter[]{filter},allFileFilterUsed);
	}

	private static void init() {
		if(STD_FILE_CHOOSER==null) {
			STD_FILE_CHOOSER=new SimpleFileChooser();
		}
	}
}